@@ -7,15 +7,16 @@ import re
 from django.core.exceptions import ValidationError
 from django.db import models
 from django.utils import formats
-from django.utils.datastructures import SortedDict
+from django.utils.dates import MONTHS
 from django.utils.text import capfirst
-from django_easyfilters.queries import date_aggregation
+from django_easyfilters.queries import date_aggregation, value_counts, numeric_range_counts
+from django_easyfilters.ranges import auto_ranges
 
 try:
     from collections import namedtuple
     FilterChoice = namedtuple('FilterChoice', 'label count params link_type')
 except ImportError:
-    # We don't use it a tuple, so this will do:
+    # We don't use it as a tuple, so this will do:
     class FilterChoice(object):
         def __init__(self, label, count, params, link_type):
             self.label, self.count, self.params, self.link_type = label, count, params, link_type
@@ -55,9 +56,9 @@ class Filter(object):
         Apply the filtering defined in params (request.GET) to the queryset qs,
         returning the new QuerySet.
         """
-        choices = list(self.chosen)
-        while len(choices) > 0:
-            lookup = self.lookup_from_choice(choices.pop())
+        chosen = list(self.chosen)
+        while len(chosen) > 0:
+            lookup = self.lookup_from_choice(chosen.pop())
             qs = qs.filter(**lookup)
         return qs
 
@@ -225,15 +226,29 @@ class SimpleQueryMixin(object):
         The order is the underlying order produced by sorting ascending on the
         DB field.
         """
-        values_counts = qs.values_list(self.field).order_by(self.field).annotate(models.Count(self.field))
-
-        count_dict = SortedDict()
-        for val, count in values_counts:
-            count_dict[val] = count
-        return count_dict
+        return value_counts(qs, self.field)
 
 
-class DrillDownMixin(object):
+class RangeFilterMixin(ChooseAgainMixin, SingleValueMixin):
+
+    # choice_type must be set to a class that provides the static method
+    # 'from_param' and instance methods 'make_lookup' and 'display', and the
+    # __cmp__ and __eq__ methods for sorting.
+    choice_type = None
+
+    def choice_from_param(self, param):
+        return self.choice_type.from_param(param)
+
+    def choices_from_params(self):
+        choices = super(RangeFilterMixin, self).choices_from_params()
+        choices.sort()
+        return choices
+
+    def lookup_from_choice(self, choice):
+        return choice.make_lookup(self.field)
+
+    def display_choice(self, choice):
+        return choice.display()
 
     def get_choices_remove(self, qs):
         # Due to drill down, if a broader param is removed, the more specific
@@ -257,20 +272,6 @@ class ValuesFilter(ChooseOnceMixin, SimpleQueryMixin, Filter):
     """
     Fallback Filter for various kinds of simple values.
     """
-    def get_values_counts(self, qs):
-        """
-        Returns a SortedDict dictionary of {value: count}.
-
-        The order is the underlying order produced by sorting ascending on the
-        DB field.
-        """
-        values_counts = qs.values_list(self.field).order_by(self.field).annotate(models.Count(self.field))
-
-        count_dict = SortedDict()
-        for val, count in values_counts:
-            count_dict[val] = count
-        return count_dict
-
     def display_choice(self, choice):
         retval = unicode(choice)
         if retval == u'':
@@ -351,55 +352,48 @@ class ForeignKeyFilter(ChooseOnceMixin, SimpleQueryMixin, RelatedObjectMixin, Fi
 
 class ManyToManyFilter(ChooseAgainMixin, RelatedObjectMixin, Filter):
 
-    def get_choices_add(self, qs):
+    def get_values_counts(self, qs):
         # It is easiest to base queries around the intermediate table, in order
         # to get counts.
         through = self.field_obj.rel.through
         rel_model = self.rel_model
 
         assert rel_model != self.model, "Can't cope with this yet..."
-
-        fkey_to_this_table = [f for f in through._meta.fields
-                              if f.rel is not None and f.rel.to is self.model][0]
-        fkey_to_other_table = [f for f in through._meta.fields
-                               if f.rel is not None and f.rel.to is rel_model][0]
+        fkey_this = [f for f in through._meta.fields
+                     if f.rel is not None and f.rel.to is self.model][0]
+        fkey_other = [f for f in through._meta.fields
+                      if f.rel is not None and f.rel.to is rel_model][0]
 
         # We need to limit items by what is in the main QuerySet (which might
         # already be filtered).
-        main_filter = {fkey_to_this_table.name + '__in':qs}
-        m2m_objs = through.objects.filter(**main_filter)
+        m2m_objs = through.objects.filter(**{fkey_this.name + '__in':qs})
 
         # We need to exclude items in other table that we have already filtered
         # on, because they are not interesting.
-        exclude_filter = {fkey_to_other_table.name + '__in': self.chosen}
-        m2m_objs = m2m_objs.exclude(**exclude_filter)
+        m2m_objs = m2m_objs.exclude(**{fkey_other.name + '__in': self.chosen})
 
         # Now get counts:
-        field_name = fkey_to_other_table.name
-        values_counts = m2m_objs.values_list(field_name).order_by(field_name).annotate(models.Count(field_name))
-
-        count_dict = SortedDict()
-        for val, count in values_counts:
-            count_dict[val] = count
+        field_name = fkey_other.name
+        return value_counts(m2m_objs, field_name)
 
+    def get_choices_add(self, qs):
+        count_dict = self.get_values_counts(qs)
         # Now, need to lookup objects on related table, to display them.
-        objs = rel_model.objects.filter(pk__in=count_dict.keys())
+        objs = self.rel_model.objects.filter(pk__in=count_dict.keys())
 
-        choices = []
-        for o in objs:
-            pk = o.pk
-            choices.append(FilterChoice(unicode(o),
-                                        count_dict[pk],
-                                        self.build_params(add=pk),
-                                        FILTER_ADD))
-        return choices
+        return [FilterChoice(unicode(o),
+                             count_dict[o.pk],
+                             self.build_params(add=o.pk),
+                             FILTER_ADD)
+                for o in objs]
 
     def get_choices_remove(self, qs):
         chosen = self.chosen
         # Do a query in bulk to get objs corresponding to choices.
         objs = self.rel_model.objects.filter(pk__in=chosen)
 
-        # We want to preserve order of items in params, so use a dict:
+        # We want to preserve order of items in params, so use the original
+        # 'chosen' list, rather than objs.
         obj_dict = dict([(obj.pk, obj) for obj in objs])
         return [FilterChoice(unicode(obj_dict[choice]),
                              None, # Don't need count for removing
@@ -413,7 +407,9 @@ class DateRangeType(object):
     all = {} # Keep a cache, so that we have unique instances
 
     def __init__(self, level, single, label, regex):
-        self.level, self.single, self.label, self.regex = level, single, label, regex
+        self.level, self.single, self.label = level, single, label
+        self.regex = re.compile((r'^(%s)$' % regex) if single else
+                                (r'^(%s)..(%s)$' % (regex, regex)))
         DateRangeType.all[(level, single)] = self
 
     def __repr__(self):
@@ -452,12 +448,14 @@ class DateRangeType(object):
             # ranges (i.e. multi) if appropriate.
             return DateRangeType.get(self.level + 1, True)
 
-YEARGROUP   = DateRangeType(1, False, 'year',  re.compile(r'^(\d{4})..(\d{4})$'))
-YEAR        = DateRangeType(1, True,  'year',  re.compile(r'^(\d{4})$'))
-MONTHGROUP  = DateRangeType(2, False, 'month', re.compile(r'^(\d{4}-\d{2})..(\d{4}-\d{2})$'))
-MONTH       = DateRangeType(2, True,  'month', re.compile(r'^(\d{4}-\d{2})$'))
-DAYGROUP    = DateRangeType(3, False, 'day',   re.compile(r'^(\d{4}-\d{2}-\d{2})..(\d{4}-\d{2}-\d{2})$'))
-DAY         = DateRangeType(3, True,  'day',   re.compile(r'^(\d{4}-\d{2}-\d{2})$'))
+
+_y, _ym, _ymd = r'\d{4}', r'\d{4}-\d{2}', r'\d{4}-\d{2}-\d{2}'
+YEARGROUP   = DateRangeType(1, False, 'year',  _y)
+YEAR        = DateRangeType(1, True,  'year',  _y)
+MONTHGROUP  = DateRangeType(2, False, 'month', _ym)
+MONTH       = DateRangeType(2, True,  'month', _ym)
+DAYGROUP    = DateRangeType(3, False, 'day',   _ymd)
+DAY         = DateRangeType(3, True,  'day',   _ymd)
 
 
 class DateChoice(object):
@@ -481,6 +479,7 @@ class DateChoice(object):
         return '<DateChoice %s %s>' % (self.range_type, self.__unicode__())
 
     def __cmp__(self, other):
+        # 'greater' means more specific.
         return cmp((self.range_type, self.values),
                    (other.range_type, other.values))
 
@@ -492,7 +491,6 @@ class DateChoice(object):
             if self.range_type == YEAR:
                 return parts[0]
             elif self.range_type == MONTH:
-                from django.utils.dates import MONTHS
                 return unicode(MONTHS[int(parts[1])])
             elif self.range_type == DAY:
                 return str(int(parts[-1]))
@@ -526,6 +524,7 @@ class DateChoice(object):
             m = drt.regex.match(param)
             if m is not None:
                 return DateChoice(drt, list(m.groups()))
+        raise ValueError()
 
     def make_lookup(self, field_name):
         # It's easier to do this all using datetime comparisons than have a
@@ -552,7 +551,9 @@ class DateChoice(object):
                 field_name + '__lt':  end_date}
 
 
-class DateTimeFilter(ChooseAgainMixin, SingleValueMixin, DrillDownMixin, Filter):
+class DateTimeFilter(RangeFilterMixin, Filter):
+
+    choice_type = DateChoice
 
     max_depth_levels = {'year': YEAR.level,
                         'month': MONTH.level,
@@ -565,23 +566,6 @@ class DateTimeFilter(ChooseAgainMixin, SingleValueMixin, DrillDownMixin, Filter)
         self.max_depth_level = self.max_depth_levels[self.max_depth]
         super(DateTimeFilter, self).__init__(*args, **kwargs)
 
-    def choice_from_param(self, param):
-        choice = DateChoice.from_param(param)
-        if choice is None:
-            raise ValueError()
-        return choice
-
-    def choices_from_params(self):
-        choices = super(DateTimeFilter, self).choices_from_params()
-        choices.sort()
-        return choices
-
-    def lookup_from_choice(self, choice):
-        return choice.make_lookup(self.field)
-
-    def display_choice(self, choice):
-        return choice.display()
-
     def get_choices_add(self, qs):
         chosen = list(self.chosen)
         range_type = None
@@ -696,8 +680,149 @@ class DateTimeFilter(ChooseAgainMixin, SingleValueMixin, DrillDownMixin, Filter)
             date_choice = DateChoice(DateRangeType.get(chosen_level, True),
                                      new_choice.values)
             retval.append(FilterChoice(date_choice.display(),
-                                       None,
-                                       None,
+                                       None, None,
                                        FILTER_DISPLAY))
         return retval
 
+
+class RangeEnd(object):
+    """
+    Simple structure to store part of a range
+    """
+    def __init__(self, value, inclusive):
+        # value is some generic value, inclusive is a bool specifying where this
+        # value is included as part of the range.
+        self.value, self.inclusive = value, inclusive
+
+
+def make_numeric_range_choice(to_python, to_str):
+    """
+    Returns a Choice class that represents a numeric choice range,
+    using the passed in 'to_python' and 'to_str' callables to do
+    conversion to/from native data types.
+    """
+
+    class NumericRangeChoice(object):
+        def __init__(self, values):
+            # Values are instances of RangeEnd
+            self.values = values
+
+        def display(self):
+            return '-'.join([str(v.value) for v in self.values])
+
+        @classmethod
+        def from_param(cls, param):
+            vals = []
+            for p in param.split('..', 1):
+                inclusive = False
+                if p.endswith('i'):
+                    inclusive = True
+                    p = p[:-1]
+
+                try:
+                    val = to_python(p)
+                    vals.append(RangeEnd(val, inclusive))
+                except ValidationError:
+                    raise ValueError()
+            return cls(vals)
+
+        def make_lookup(self, field_name):
+            if len(self.values) == 1:
+                return {field_name: self.values[0].value}
+            else:
+                start, end = self.values[0], self.values[1]
+                return {field_name + '__gt' + ('e' if start.inclusive else ''): start.value,
+                        field_name + '__lt' + ('e' if end.inclusive else ''): end.value}
+
+        def __unicode__(self):
+            return '..'.join([to_str(v.value) + ('i' if v.inclusive else '')
+                              for v in self.values])
+
+        def __repr__(self):
+            return '<NumericChoice %s>' % self.__unicode__()
+
+        def __cmp__(self, other):
+            # 'greater' means more specific.
+            if other is None:
+                return cmp(self.values, None)
+            else:
+                if len(self.values) != len(other.values):
+                    # one value is more specific than two
+                    return -cmp(len(self.values), len(other.values))
+                elif len(self.values) == 1:
+                    return 0
+                else:
+                    # Larger difference means less specific
+                    return -cmp(self.values[1].value - self.values[0].value,
+                                other.values[1].value - other.values[0].value)
+
+    return NumericRangeChoice
+
+
+class NumericRangeFilter(RangeFilterMixin, Filter):
+
+    def __init__(self, field, model, params, **kwargs):
+        self.max_links = kwargs.pop('max_links', 5)
+        self.drilldown = kwargs.pop('drilldown', True)
+        self.ranges = kwargs.pop('ranges', None)
+        field_obj = model._meta.get_field(field)
+        self.choice_type = make_numeric_range_choice(field_obj.to_python, str)
+        super(NumericRangeFilter, self).__init__(field, model, params, **kwargs)
+
+    def display_choice(self, c):
+        if self.ranges is None:
+            return c.display()
+        if len(c.values) == 1:
+            return c.display()
+        for r in self.ranges:
+            if r[0] == c.values[0].value and r[1] == c.values[1].value:
+                # found a match in ranges
+                if len(r) > 2:
+                    # found a label, so use it
+                    return r[2]
+                else:
+                    return c.display()
+        return c.display()
+
+    def get_choices_add(self, qs):
+        chosen = list(self.chosen)
+        range_type = None
+
+        if not self.drilldown and len(chosen) > 0:
+            return []
+
+        all_vals = qs.values_list(self.field).distinct()
+
+        num = all_vals.count()
+
+        choices = []
+        if num <= self.max_links:
+            val_counts = value_counts(qs, self.field)
+            for v, count in val_counts.items():
+                choice = self.choice_type([RangeEnd(v, True)])
+                choices.append(FilterChoice(self.display_choice(choice),
+                                            count,
+                                            self.build_params(add=choice),
+                                            FILTER_ADD))
+        else:
+            if self.ranges is None:
+                val_range = qs.aggregate(lower=models.Min(self.field),
+                                         upper=models.Max(self.field))
+                ranges = auto_ranges(val_range['lower'], val_range['upper'], self.max_links)
+            else:
+                ranges = self.ranges
+
+            val_counts = numeric_range_counts(qs, self.field, ranges)
+            for i, (vals, count) in enumerate(val_counts.items()):
+                # For the lower bound, we make it inclusive only if it the first
+                # choice. The upper bound is always inclusive. This gives
+                # filters that behave sensibly e.g. with 10-20, 20-30, 30-40,
+                # the first will include 10 and 20, the second will exlude 20.
+                choice = self.choice_type([RangeEnd(vals[0], i == 0),
+                                           RangeEnd(vals[1], True)])
+                choices.append(FilterChoice(self.display_choice(choice),
+                                            count,
+                                            self.build_params(add=choice),
+                                            FILTER_ADD))
+
+        return choices
