@@ -14,6 +14,19 @@ def non_breaking_spaces(val):
     return mark_safe(u'&nbsp;'.join(escape(part) for part in val.split(u' ')))
 
 
+class cachedproperty(object):
+    """
+    Decorator that creates converts a method with a single
+    self argument into a property cached on the instance.
+    """
+    def __init__(self, func):
+        self.func = func
+
+    def __get__(self, instance, type):
+        res = instance.__dict__[self.func.__name__] = self.func(instance)
+        return res
+
+
 class FilterSet(object):
 
     template = """
@@ -31,6 +44,7 @@ class FilterSet(object):
 {% endfor %}
 </div>
 """
+    title_fields = None
 
     def __init__(self, queryset, params):
         self.params = params
@@ -38,14 +52,24 @@ class FilterSet(object):
         self.filters = self.setup_filters()
         self.qs = self.apply_filters(queryset)
 
+    @cachedproperty
+    def title(self):
+        return self.make_title()
+
+    def get_filter_choices(self, filter_field):
+        if not hasattr(self, '_cached_filter_choices'):
+            self._cached_filter_choices = dict((f.field, f.get_choices(self.qs))
+                                               for f in self.filters)
+        return self._cached_filter_choices[filter_field]
+
     def apply_filters(self, queryset):
         for f in self.filters:
             queryset = f.apply_filter(queryset)
         return queryset
 
-    def render_filter(self, filter_, qs):
+    def render_filter(self, filter_):
         field_obj = self.model._meta.get_field(filter_.field)
-        choices = filter_.get_choices(qs)
+        choices = self.get_filter_choices(filter_.field)
         ctx = {'filterlabel': capfirst(field_obj.verbose_name)}
         ctx['choices'] = [dict(label=non_breaking_spaces(c.label),
                                url=u'?' + c.params.urlencode() \
@@ -59,7 +83,7 @@ class FilterSet(object):
         return template.Template(self.template)
 
     def render(self):
-        return mark_safe(u'\n'.join(self.render_filter(f, self.qs) for f in self.filters))
+        return mark_safe(u'\n'.join(self.render_filter(f) for f in self.filters))
 
     def get_fields(self):
         return self.fields
@@ -99,5 +123,15 @@ class FilterSet(object):
             filters.append(klass(field_name, self.model, self.params, **opts))
         return filters
 
+    def make_title(self):
+        if self.title_fields is None:
+            title_fields = [filter_.field for filter_ in self.filters]
+        else:
+            title_fields = self.title_fields
+        return u", ".join(c.label
+                          for f in title_fields
+                          for c in self.get_filter_choices(f)
+                          if c.link_type != FILTER_ADD)
+
     def __unicode__(self):
         return self.render()
